"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/index.ts
var src_exports = {};
__export(src_exports, {
  init: () => init3
});
module.exports = __toCommonJS(src_exports);

// src/callbacks.ts
var pendingRequests = {};
var uniqueId = 0;
var getUniqueId = () => {
  uniqueId++;
  return uniqueId;
};
var makeRequest = (type, args) => {
  const requestId = type + "--" + getUniqueId();
  window.top?.postMessage(
    {
      type,
      payload: args,
      requestId
    },
    "*"
  );
  return new Promise((resolve, reject) => {
    pendingRequests[requestId] = { resolve, reject };
    const maxDelay = 1e3 * 5;
    window.setTimeout(() => {
      delete pendingRequests[requestId];
      reject(new Error("Timeout"));
    }, maxDelay);
  });
};
var callbackFunctions = {
  onUpdateMetadata: (metadata) => makeRequest("onUpdateMetadata", { metadata }),
  onNavigateToPath: (path) => makeRequest("onNavigateToPath", { path }),
  onUpdateContent: (content) => makeRequest("onUpdateContent", { content }),
  onRequestGitHubData: (path, params) => makeRequest("onRequestGitHubData", { path, params }),
  onRequestGitHubEndpoint: (route, parameters) => makeRequest("onRequestGitHubEndpoint", { route, parameters }),
  onStoreGet: (key) => makeRequest("onStoreGet", { key }),
  onStoreSet: (key, value) => makeRequest("onStoreSet", { key, value }),
  onRequestBlocksRepos: (params) => makeRequest("onRequestBlocksRepos", { params })
};
var callbackFunctionsInternal = {
  ...callbackFunctions,
  private__onFetchInternalEndpoint: (path, params) => makeRequest("private__onFetchInternalEndpoint", { path, params })
};
var init = () => {
  const onMessage = async (event) => {
    const { data } = event;
    if (!data.requestId)
      return;
    const request = pendingRequests[data.requestId];
    if (!request)
      return;
    delete pendingRequests[data.requestId];
    if (data.error) {
      request.reject(data.error);
    } else {
      request.resolve(data.response);
    }
  };
  addEventListener("message", onMessage);
};

// src/bundle.ts
var import_react = __toESM(require("react"));
var ReactJSXRuntime = __toESM(require("react/jsx-runtime"));
var import_react_dom = __toESM(require("react-dom"));
var import_client = __toESM(require("react-dom/client"));
var PrimerReact = __toESM(require("@primer/react"));
var elements = [];
var findBlockBundleName = (bundle) => {
  let blockBundleName;
  bundle.forEach((asset) => {
    if (asset.name.endsWith(".js")) {
      blockBundleName = asset.content.match(/var ([^=]+)=/)?.[1];
    }
  });
  if (!blockBundleName)
    throw new Error("couldn't find block bundle name");
  return blockBundleName;
};
var loadReactContent = (content) => {
  return `
var BlockBundle = ({ React, ReactJSXRuntime, ReactDOM, ReactDOMClient, PrimerReact }) => {
  function require(name) {
    switch (name) {
      case "react":
        return React;
      case "react/jsx-runtime":
        return ReactJSXRuntime;
      case "react-dom":
        return ReactDOM;
      case "react-dom/client":
        return ReactDOMClient;
      case "@primer/react":
      case "@primer/components":
        return PrimerReact;
      default:
        console.log("no module '" + name + "'");
        return null;
    }
  }
${content}
  return BlockBundle;
};`;
};
var unloadBundle = () => {
  for (const el of elements || []) {
    document.body.removeChild(el);
  }
  elements.splice(0, elements.length);
};
var loadBlockBundle = (blockBundleName, bundle) => {
  bundle.forEach((asset) => {
    if (asset.name.endsWith(".css")) {
      const cssElement = document.createElement("style");
      cssElement.textContent = asset.content;
      elements.push(cssElement);
    } else if (asset.name.endsWith(".js")) {
      let content;
      if (blockBundleName === "BlockBundle") {
        content = loadReactContent(asset.content);
      } else if (blockBundleName === "VanillaBlockBundle") {
        content = asset.content;
      } else {
        throw new Error(`unknown block bundle name '${blockBundleName}'`);
      }
      const jsElement = document.createElement("script");
      jsElement.textContent = content;
      elements.push(jsElement);
    }
  });
  for (const el of elements) {
    document.body.appendChild(el);
  }
};
var root;
var makeReactSetBlockProps = () => {
  const Block = window.BlockBundle({
    React: import_react.default,
    ReactJSXRuntime,
    ReactDOM: import_react_dom.default,
    ReactDOMClient: import_client.default,
    PrimerReact
  }).default;
  const BlockComponent = ({ block, context }) => {
    const { owner, repo, id, type } = block;
    const hash = encodeURIComponent(
      JSON.stringify({ block: { owner, repo, id, type }, context })
    );
    return import_react.default.createElement("iframe", {
      src: `/#${hash}`,
      sandbox: "allow-scripts allow-same-origin allow-forms allow-downloads",
      style: {
        width: "100%",
        height: "100%",
        border: 0
      }
    });
  };
  return (props) => {
    const WrappedBlockComponent = (nestedProps) => {
      let context = {
        ...props.context,
        ...nestedProps.context
      };
      const parentRepo = [props.context.owner, props.context.repo].join("/");
      const childRepo = [context.owner, context.repo].join("/");
      const isSameRepo = parentRepo === childRepo;
      if (!isSameRepo) {
        context.sha = nestedProps.context.sha || "HEAD";
      }
      return import_react.default.createElement(BlockComponent, {
        ...nestedProps,
        context
      });
    };
    if (!root) {
      root = import_client.default.createRoot(document.getElementById("root"));
    }
    root.render(
      import_react.default.createElement(
        PrimerReact.ThemeProvider,
        {},
        import_react.default.createElement(
          PrimerReact.BaseStyles,
          { style: { width: "100%", height: "100%" } },
          import_react.default.createElement(Block, {
            ...props,
            BlockComponent: WrappedBlockComponent
          })
        )
      )
    );
  };
};
var makeVanillaSetBlockProps = () => {
  return (props) => {
    window.VanillaBlockBundle.default(props);
  };
};
var makeSetBlockProps = (blockBundleName) => {
  if (blockBundleName === "BlockBundle") {
    return makeReactSetBlockProps();
  } else if (blockBundleName === "VanillaBlockBundle") {
    return makeVanillaSetBlockProps();
  } else {
    throw new Error(`unknown block bundle name '${blockBundleName}'`);
  }
};
var loadBundle = async (bundle, block, loadDevServerBlock) => {
  if (bundle.length === 0) {
    if (loadDevServerBlock) {
      return await loadDevServerBlock(block);
    } else {
      throw new Error("empty bundle but no loadDevServerBlock");
    }
  } else {
    const blockBundleName = findBlockBundleName(bundle);
    loadBlockBundle(blockBundleName, bundle);
    return makeSetBlockProps(blockBundleName);
  }
};

// src/events.ts
var init2 = (loadDevServerBlock) => {
  let bundle = void 0;
  let setBlockProps;
  let props;
  const onMessage = async (event) => {
    const { data } = event;
    if (data.type !== "setProps")
      return;
    if ("bundle" in data.props) {
      setBlockProps = void 0;
      unloadBundle();
      bundle = data.props.bundle;
    } else if (data.props.props) {
      props = data.props.props;
    }
    if (!setBlockProps && bundle && props) {
      setBlockProps = await loadBundle(bundle, props.block, loadDevServerBlock);
    }
    const root2 = document.getElementById("root");
    if (root2.children.length === 1 && (root2.children[0].id === "__loading__" || root2.children[0].id === "__not_found__")) {
      root2.removeChild(root2.children[0]);
    }
    if (bundle === null || !setBlockProps) {
      const div = document.createElement("div");
      div.id = bundle === null ? "__not_found__" : "__loading__";
      div.style.width = "100%";
      div.style.height = "100%";
      div.style.display = "flex";
      div.style.alignItems = "center";
      div.style.justifyContent = "center";
      div.style.color = "#ddd";
      div.style.fontStyle = "italic";
      const text = document.createTextNode(
        bundle === null ? "Block not found" : "Loading..."
      );
      div.appendChild(text);
      root2.appendChild(div);
    } else {
      const wrappedSetBlockProps = (props2) => {
        if (!setBlockProps)
          return;
        const isInternal = props2.block.owner === "githubnext";
        const filteredCallbackFunctions = isInternal ? callbackFunctionsInternal : callbackFunctions;
        const onUpdateContent = (content) => {
          props2 = { ...props2, content };
          wrappedSetBlockProps(props2);
          filteredCallbackFunctions["onUpdateContent"](content);
        };
        setBlockProps({
          ...props2,
          ...filteredCallbackFunctions,
          onUpdateContent
        });
      };
      wrappedSetBlockProps(props);
    }
  };
  addEventListener("message", onMessage);
  const onLoad = () => {
    window.top?.postMessage(
      {
        type: "loaded",
        hash: window.location.hash
      },
      "*"
    );
  };
  onLoad();
  addEventListener("hashchange", onLoad);
};

// src/index.ts
var init3 = (loadDevServerBlock) => {
  init();
  init2(loadDevServerBlock);
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  init
});
